% useful.spc - interface specifications for `useful' abstractions
%
% This file was derived from PCLU distribution (as of 11/18/93), and
% has been modified by Department of Mathematical and Computing
% Sciences, Tokyo Institute of Technology.
%
% Conditions for using, copying, modifying, and distributing this
% file are the same as those of PCLU.
%
% $Id: useful.spc,v 2.0 1996/06/10 09:02:39 ushijima Exp $


%  Copyright	Massachusetts Institute of Technology     1989
% Copyright Massachusetts Institute of Technology 1980, 1989
% Copyright Massachusetts Institute of Technology 1981, 1989
% Copyright Massachusetts Institute of Technology 1982, 1989


% The following table shows the file in PCLU distribution from which the
% interface specification for each abstraction was derived.
%
%	====================================================
%	Abstraction name	File name in PCLU
%	----------------------------------------------------
%	_all_file_names_	lib/clu/_all_file_names_.clu
%	_change_mode		lib/etc.spc
%	_change_owner		lib/etc.spc
%	_dir_ent_size		lib/etc.spc
%	_exit			lib/etc.spc
%	_file_access		lib/etc.spc
%	_file_exists		lib/etc.spc
%	_file_facts		lib/etc.spc
%	_file_owner		lib/etc.spc
%	_gc_copy		lib/clu/gc_copy.clu
%	_get_pname		lib/etc.spc
%	_get_runtime		lib/etc.spc
%	_get_version		lib/etc.spc
%	_get_xjname		lib/etc.spc
%	_host_name		lib/etc.spc
%	_longsub		lib/clu/_longsub.clu
%	_match_name_spec	lib/clu/_all_file_names_.clu
%	_qindexc		lib/clu/_qindexc.clu
%	_real_time		lib/etc.spc
%	_restar			lib/clu/_all_file_names_.clu
%	_signal			lib/_signal.spc
%	_sleep			lib/etc.spc
%	_stop			lib/etc.spc
%	_system_root		lib/etc.spc
%	_time_format		lib/clu/_time_format.clu
%	_unquote		lib/clu/_unquote.clu
%	all_file_names		lib/clu/all_file_names.clu
%	array_replace		lib/clu/array_replace.clu
%	connected_dir		lib/etc.spc
%	create_directory	lib/etc.spc
%	delete_directory	lib/etc.spc
%	delete_expunge		lib/clu/delete_expunge.clu
%	file_date		lib/etc.spc
%	file_exists		lib/etc.spc
%	file_name_fill		sys/clu/file_name_fill
%	gc			lib/etc.spc
%	gc_copy			lib/clu/gc_copy.clu
%	gc_copy_to		lib/clu/gc_copy.clu
%	gc_dump			lib/file_exists.spc
%	gc_read			lib/file_exists.spc
%	get_argv		lib/etc.spc
%	get_jcl			lib/etc.spc
%	hash			lib/table.spc
%	hash_int		lib/table.spc
%	hash_obj		lib/table.spc
%	host_address		lib/etc.spc
%	i_and			lib/mint.spc
%	i_first1		lib/mint.spc
%	i_get			lib/mint.spc
%	i_get1			lib/mint.spc
%	i_hparse		lib/clu/hex.clu
%	i_hunparse		lib/clu/hex.clu
%	i_last1			lib/mint.spc
%	i_not			lib/mint.spc
%	i_oparse		lib/clu/octal.clu
%	i_or			lib/mint.spc
%	i_ounparse		lib/clu/octal.clu
%	i_rotate		lib/mint.spc
%	i_set			lib/mint.spc
%	i_set1			lib/mint.spc
%	i_shift			lib/mint.spc
%	i_xor			lib/mint.spc
%	isqrt			lib/clu/isqrt.clu
%	link_file		lib/etc.spc
%	maybe			lib/maybe.spc
%	open_append		lib/clu/open_append.clu
%	open_read		lib/clu/open_read.clu
%	open_write		lib/clu/open_write.clu
%	quit_			lib/etc.spc
%	run_time		lib/clu/run_time.clu
%	set_connected_dir	lib/etc.spc
%	sleep			lib/etc.spc
%	symlink_file		lib/etc.spc
%	table			lib/table.spc
%	time			lib/clu/time.clu
%	unify_spacing		lib/clu/unify_spacing.clu
%	unix_cmd		lib/etc.spc
%	--------------------------------------------------


% Data abstractions

% _Signal

_signal = cluster is set, get, unset, set_message, get_message

% Operations

set = proc(SIG: int, HOLD: bool) signals(bad_code)
	% set to count occurrences of given signal if hold = FALSE
	% place signal on hold if hold = TRUE
	% NOTE: signal 8 (SIGFPE) should be avoided if real arithmetic is used
	% NOTE: signals 2 (SIGINT), 3 (SIGQUIT), 4 (SIGILL), 10 (SIGBUS),
	% 11 (SIGSEGV), and 18 (SIGTSTP) should not be set until after a
	% call to stream/_chan primary_input/output
	end set

unset = proc(SIG: int) signals(bad_code)
	% restore signal to its orignal setting
	end unset

get = proc(SIG: int) returns(int) signals(bad_code)
	% get number of occurrences of signal since last get call
	end get

set_message = proc(SIG: int, MSG: string) signals(bad_code)
	% set message to be output (on primary output) on every
	% occurrence of signal
	end set_message

get_message = proc(SIG: int) returns(string) signals(bad_code)
	% get message to be output for signal
	end get_message

end _signal


% Maybe

maybe = cluster[T: type] is none, make, exists, value, equal, similar, copy,
	_gcd

% Overview
%
% This data type is similar to oneof[exists: T, none: null],
% but it doesn't require any additional storage.
% However, its operations also aren't inlined by the compiler and some
% instantiations can cause maybe$make to signal failure.
%
% Restrictions to guarantee that make doesn't signal failure:
%	1. The type t is not an instantiation of maybe.
%	   I.e., maybe[maybe[...]]$make can signal failure.
%	   If T is a cluster whose primitive rep is an instantiation of
%	   maybe, maybe[T] is not supported.  A useful convention is not
%	   to use reps that are instantiations of maybe.
%	2. Do not call maybe[int]$make(Smallest_Integer), which signals
%	   failure.  Maybe uses the smallest integer to represent the "none"
%	   value.
%	3. maybe[real] is a bad idea, since the smallest integer is the
%	   same as a real number slightly less than zero.
%
% Note: Under UNIX, the smallest integer is a kernel address, so maybe[t]
% is guaranteed to work when t is an array, record, sequence, or struct.
% If t is an abstract type whose rep is guaranteed to work, maybe[t] is
% also guaranteed to work provided that t is not an intantiation of maybe.
% Other instantiations may fail by signalling failure when make is called.

% Operations

none = proc() returns(maybe[T])
	% return the object with the NONE tag
	end none

make = proc(X: T) returns(maybe[T])
	% return a maybe with tag EXISTS and value X
	end make

exists = proc(M: maybe[T]) returns(bool)
	% test if M has tag EXISTS
	end exists

value = proc(M: maybe[T]) returns(T) signals(wrong_tag)
	% return the value of M if the tag is EXISTS, else signal wrong_tag
	end value

equal = proc(M1, M2: maybe[T]) returns(bool)
	where T has equal: proctype(T, T) returns(bool)
	% true if both have tag NONE or
	% both have tag EXISTS and the values are t$equal
	end equal

similar = proc(M1, M2: maybe[T]) returns(bool)
	  where T has similar: proctype(T, T) returns(bool)
	% true if both have tag NONE or
	% both have tag EXISTS and the values are t$similar
	end similar

copy = proc(M: maybe[T]) returns(maybe[T])
       where T has copy: proctype(T) returns(T)
	% if M has tag NONE, return it
	% else return a maybe with tag EXISTS and value t$copy(value(m))
	end copy

_gcd = proc(M: maybe[T], tab: gcd_tab) returns(int)
       where T has _gcd: proctype(T, gcd_tab) returns(int)
	end _gcd

end maybe


% Table

table = cluster[KEY, VAL: type] is create, flush, cond_flush, pcond_flush,
	bind, mbind, alter, lookup, mlookup, get_pair, exists, remove, delete,
	rehash, elements, count, empty, sizes

% Overview
%
% A mutable mapping from objects of one arbitrary type to another

% Operations

create = proc(SIZE: int, HASHER: proctype(KEY, int) returns(int),
	      COMPER: proctype(KEY, KEY) returns(bool))
	 returns(table[KEY, VAL]) signals(bad_size)
	% Create a table.  SIZE indicates the number of hashing buckets to use:
	%	1 <= SIZE
	% HASHER is the hashing function:
	%	0 <= HASHER(key, SIZE) <= SIZE - 1
	% for all keys.  COMPER is an equivalence relation on keys.  For any K,
	% K' is the key in the table such that COMPER(K, K') holds.  K' exists
	% if and only if it has a value bound to it. For things to work, it
	% must be the case that
	%	COMPER(K, K')  implies  HASHER(K, SIZE) = HASHER(K', SIZE)
	end create

flush = proc(T: table[KEY, VAL])
	% Removes all bindings
	end flush

cond_flush = proc(T: table[KEY, VAL], PRED: proctype(VAL) returns(bool))
	% Flushes all (key, val) pairs for which PRED(val) is true
	end cond_flush

pcond_flush = proc[U: type](T: table[KEY, VAL],
			    PRED: proctype(VAL, U) returns(bool), THING: U)
	% Flushes all (key, val) pairs for which PRED(val, THING) is true
	end pcond_flush

bind = proc(T: table[KEY, VAL], K: KEY, V: VAL) signals(exists)
	% Bind the value V to the key K.  Signal EXISTS if K' exists.
	end bind

mbind = proc(T: table[KEY, VAL], K: KEY, V: VAL) returns(val)
	% If K' exists, return the value bound to it, else bind V to K and
	% return V.
	end mbind

alter = proc(T: table[KEY, VAL], K: KEY, V: VAL)
	% Bind the value V to the key K', if it exists, else to K.
	end alter

lookup = proc(T: table[KEY, VAL], K: KEY) returns(VAL) signals(not_found)
	% Get the value bound to K'.  Signal NOT_FOUND if K' does not exist.
	end lookup

mlookup = proc(T: table[KEY, VAL], K: KEY, DEFV: VAL) returns(VAL)
	% Get the value bound to K'.  Return DEFV if K' does not exist.
	end mlookup

get_pair = proc(T: table[KEY, VAL], K: KEY) returns(KEY, VAL)
	   signals(not_found)
	% Get the value bound to K', and return both K' and the value.
	% Signal NOT_FOUND if K' does not exist.
	end get_pair

exists = proc(T: table[KEY, VAL], K: KEY) returns(bool)
	% Test if the key K' exists.
	end exists

remove = proc(T: table[KEY, VAL], K: KEY) returns(VAL) signals(not_found)
	% Remove the value bound to key K' and return it.  Signal NOT_FOUND if
	% K' does not exist.  Afterwards K' does not exist.
	end remove

delete = proc(T: table[KEY, VAL], K: KEY)
	% Remove the value bound to key K', if any.  Afterwards K' does not
	% exist.
	end delete

rehash = proc(T: table[KEY, VAL], HASHER: proctype(KEY, int) returns(int))
	% Rebuild the hash table with a new HASHER.
	end rehash

elements = iter(T: table[KEY, VAL]) yields(KEY, VAL)
	% Yield all (key, value) pairs in the table in random order
	end elements

count = proc(T: table[KEY, VAL]) returns(int)
	% Return the total number of (key, value) pairs
	end count

empty = proc(T: table[KEY, VAL]) returns(bool)
	% Test if table is empty
	end empty

sizes = proc(T: table[KEY, VAL]) returns(array[int])
	% Return an array of integers with element i being the number of
	% buckets with i (key, val) pairs
	end sizes

end table


% Time

% time.spc - interface specification of time


time = cluster is create, get_days, get_hours, get_minutes, get_seconds,
       get_millis, get_micros, get_all, add, sub, mul, div, average, lt, le,
       ge, gt, parse, unparse, t2r, format, print, equal, similar, copy,
       encode, decode, _gcd

% Overview
%
% An object of the `time' type is immutable and represents time.  A `time'
% object is viewed as consisting of six integer components:
%
% `days'
%	greater than or equal to zero.
%
% `hours'
%	in the range [0, 23].
%
% `minutes'
%	in the range [0, 59].
%
% `seconds'
%	1,000,000 microseconds in the range [0, 59].
%
% `milliseconds'
%	1,000 microseconds in the range [0, 999].
%
% `microseconds'
%	in the range [0, 999].
%
% A `time' object is also viewed as a single integer that represents time value
% measured in microseconds.  That is, if a `time' object consists of DAYS,
% HOURS, MINS, SECS, MILLIS, and MICROS for the `days', `hours', `minutes',
% `seconds', `milliseconds', and `microseconds', respectively, it is also viewd
% as representing M(DAYS, HOURS, MINS, SECS, MILLIS, MICROS) microseconds,
% where M is defined as:
%   M(U, V, W, X, Y, Z)
%	= ((((U * 24 + V) * 60 + W) * 60 + X) * 1000 + Y) * 1000 + Z.
% Arithmetic and comparison operations such as `add', `sub', `lt', and `equal'
% are defined in this single integer view.
%
% Operations that return a `time' object in the normal condition signals
% `overflow' when the resulting time would be so large that the `days'
% component could not be represented as an `int' object, unless otherwise
% specified.

% Operations

create = proc(DAYS, HOURS, MINS, SECS, MILLIS, MICROS: int) returns(time)
	 signals(bad_format, overflow)
	% effects  Signals `bad_format' if at least one argument are negative;
	%	   otherwise returns the time that represents
	%	   M(DAYS, HOURS, MINS, SECS, MILLIS, MICROS) microseconds.
	end create
    
get_days = proc(T: time) returns(int)
	% effects  Returns the `days' component of T.
	end get_days
    
get_hours = proc(T: time) returns(int)
	% effects  Returns the `hours' component of T.
	end get_hours
    
get_minutes = proc(T: time) returns(int)
	% effects  Returns the `minutes' components of T.
	end get_minutes
    
get_seconds = proc(T: time) returns(int)
	% effects  Returns the `seconds' component of T.
	end get_seconds
    
get_millis = proc(T: time) returns(int)
	% effects  Returns the `milliseconds' component of T.
	end get_millis
    
get_micros = proc(T: time) returns(int)
	% effects  Returns the `micorseconds' component of T.
	end get_micros
    
get_all = proc(T: time) returns(int, int, int, int, int, int)
	% effects  Returns the `days', `hours', `minutes', `seconds',
	%	   `milliseconds', and `microseconds' components, in this
	%	   order, of T.
	end get_all
    
add = proc(T1, T2: time) returns(time) signals(overflow)
	% effects  Views both T1 and T2 as single integers.  Returns the `time'
	%	   object that represents the sum of these two integers.
	end add
    
sub = proc(T1, T2: time) returns(time) signals(negative_time)
	% effects  Views both T1 and T2 as single integers.  Signals
	%	   `negative_time' if the integer represented by T1 is less
	%	   than the integer represented by T2; otherwise returns the
	%	   `time' object that represents 
	end sub
    
mul = proc(T: time, N: int) returns(time)
      signals(overflow, negative_multiplier)
	end mul

div = proc(T: time, N: int) returns(time) signals(bad_divisor)
	end div

average = proc(T: time, N: int) returns(time) signals(bad_divisor)
	end average
    
lt = proc(T1, T2: time) returns(bool)
	end lt
    
le = proc(T1, T2: time) returns(bool)
	end le
    
ge = proc(T1, T2: time) returns(bool)
	end ge
    
gt = proc(T1, T2: time) returns(bool)
	end gt
    
parse = proc(S: string) returns(time) signals(bad_format, overflow)
	% string must have form "days:hours:mins:secs.fraction"
	% although leading zero fields need not be present.
	end parse
    
unparse = proc(T: time) returns(string)
	end unparse

t2r = proc(T: time) returns(real) signals(overflow)
	% returns time in seconds
	end t2r
    
format = proc(T: time, FRAC_WIDTH: int) returns(string)
	% returns  "days:hours:mins:secs.fraction" with leading zero
	% fields (above secs) suppressed; second arg indicates number
	% of fractional digits (truncated)
	end format
    
print = proc(T: time, PST: pstream)
	end print
    
equal = proc(T1, T2: time) returns(bool)
	end equal
    
similar = proc(T1, T2: time) returns(bool)
	% effects  Equivalent to the `equal' operation.
	end similar
    
copy = proc(T: time) returns(time)
	end copy
    
encode = proc(T: time, IST: istream) signals(not_possible(string))
	end encode

decode = proc(IST: istream) returns(time)
	 signals(end_of_file, not_possible(string))
	end decode

_gcd = proc(T: time, TAB: gcd_tab) returns(int)
	end _gcd

end time


% Procedural/control abstractions


_all_file_names_ = iter(FS: string) yields(file_name)
		   signals(bad_format, not_possible(string))
	end _all_file_names_

_change_mode = proc(FN: file_name, MODE: int) signals(not_possible(string))
	% effects  Invokes the `chmod' system call with the given arguments.
	%	   Terminates in the normal condition if the call was
	%	   successful; otherwise signals `not_possible'.
	end _change_mode

_change_owner = proc(FN: file_name, OWNER, GROUP: int)
		signals(not_possible(string))
	% effects  Invokes the `chown' system call with the given arguments.
	%	   Terminates in the normal condition if the call was
	%	   successful; otherwise signals `not_possible'.
	end _change_owner

_dir_ent_size = proc() returns(int)
	% effects  Returns the maximum size of a name of a directory entry.
	end _dir_ent_size

_exit = proc(STATUS: int)
	% effects  Does some cleanup, e.g., flushing unwritten buffer contents
	%	   to files, and terminates the current process with STATUS as
	%	   an exit status.
	end _exit

_file_access = proc(FS: string, MODE: int) returns(bool)
	% effects  Invokes the `access' system call with the given arguments.
	%	   Terminates in the normal condition if the call was
	%	   successful; otherwise signals `not_possible'.
	end _file_access

_file_exists = proc(FS: string, FTYPE: int) returns(bool)
	       signals(not_possible(string))
    	% test if file is of given type
    	% directory = 4, char special = 2, block special = 6, regular = 8
    	% multiplexed char special = 3, multiplexed block special = 7
	end _file_exists

_file_facts = proc(FN: file_name) returns(int, int)
	      signals(not_possible(string))
    	% return number of links, file mode of given file
	end _file_facts

_file_owner = proc(FN: file_name) returns(int, int)
	      signals(not_possible(string))
    	% return user id and group id of given file
	end _file_owner

_gc_copy = proc(T: record[tab: table[_obj, _obj], count: int], X: _obj)
	   returns(_obj)
	end _gc_copy

% _gc_read = 
%	end _gc_read

_get_pname = proc() returns(string)
    	% return program name with directory, if any
	end _get_pname

_get_runtime = proc() returns(int, int, int)
    	% return seconds, milliseconds, microseconds of user+system time
    	% for current process
	end _get_runtime

_get_version = proc() returns(int)
    	% return program version number (set by #version option of LINK)
	end _get_version

_get_xjname = proc() returns(string)
    	% return program name with directory removed
	end _get_xjname

_host_name = proc() returns(string)
    	% returns host name
	end _host_name

_longsub = proc(X1, X2, X3, Y1, Y2, Y3: int) returns(int, int, int)
	end _longsub

_match_name_spec = proc(OFF: int, BUF: _bytevec, NM: string) returns(int)
	end _match_name_spec

_qindexc = proc(C: char, S: string) returns(int)
	end _qindexc

_real_time = proc() returns(int)
    	% return real time in milliseconds since unknown event in past
	end _real_time

_restar = proc(S: string) returns(string)
	end _restar

_sleep = proc(MILLIS: int)
    	% sleep for MILLIS milliseconds.
	end _sleep

_stop = proc()
    	% force SIGTSTP signal (interrupt) on this process
    	% to stop in a continuable fashion
	end _stop

_system_root = proc() returns(string)
    	% returns CLU system root directory name (no trailing slash)
	end _system_root

_time_format = proc(SEC, MSEC, USEC: int) returns(string)
	end _time_format

_unquote = proc(S: string) returns(string)
	end _unquote

all_file_names = iter(FS: string) yields(file_name)
		 signals(bad_format, none, not_possible(string))
	end all_file_names

array_replace = proc[T: type](A1: array[T], B1, W1: int,
			      A2: array[T], B2, W2: int)
		signals(bounds, negative_size)
	end array_replace

connected_dir = proc() returns(string)
    	% Get connected directory
	end connected_dir

create_directory = proc(FN: file_name, MODE: int) signals(not_possible(string))
	end create_directory

delete_directory = proc(FN: file_name) signals(not_possible(string))
	end delete_directory

delete_expunge = proc(FN: file_name) signals(not_possible(string))
	end delete_expunge

file_date = proc(FN: file_name, READ: bool) returns(date)
	    signals(not_possible(string))
	end file_date

file_exists = proc(FN: file_name) returns(bool) signals(not_possible(string))
	end file_exists

file_name_fill = proc(FN: file_name, DSUFFIX: string) returns(file_name)
	end file_name_fill

gc = proc() returns(int)
    	% returns number of (long) words reclaimed
	end gc

gc_copy = proc[T: type](X: T) returns(T)
	end gc_copy

gc_copy_to = proc[T: type](X: T, LEAVES: array[_obj]) returns(T)
	end gc_copy_to

gc_dump = proc[T: type](X: T, FN: file_name) returns(file_name)
	  signals(not_possible(string))
	  where T has _gcd: proctype(T, gcd_tab) returns(int)
	end gc_dump

gc_read = proc[T: type](FN: file_name) returns(T) signals(not_possible(string))
	end gc_read

get_argv = proc() returns(sequence[string])
    	% returns command line arguments, not including program name
	end get_argv

get_jcl = proc() returns(string)
    	% returns command line, not including program name, first time called
	end get_jcl

hash = proc(S: string, SIZE: int) returns(int)
	% A hashing function for strings.
	end hash

hash_int = proc(I, SIZE: int) returns(int)
	% A hashing function for integers.
	end hash_int

hash_obj = proc(O: _obj, MAX: int) returns(int)
	end hash_obj

host_address = proc(HOST: string) returns(int, int)
	       signals(not_found, bad_address)
    	% returns upper and lower halves of host address
	end host_address

i_and = proc(I, J: int) returns(int) signals(overflow)
	end i_and

i_first1 = proc(I: int) returns(int) signals(none)
	% return number of first 1 bit
	end i_first1

i_get = proc(I, START, CNT: int) returns(int)
	signals(bounds, illegal_size, overflow)
	% low order bit is numbered 0
	end i_get

i_get1 = proc(I, BIT: int) returns(bool) signals(bounds)
	% low order bit is numbered 0
	end i_get1

i_hparse = proc(S: string) returns(int) signals(bad_format, overflow)
	end i_hparse

i_hunparse = proc(I: int) returns(string)
	end i_hunparse

i_last1 = proc(I: int) returns(int) signals(none)
	% return number of last 1 bit
	end i_last1

i_not = proc(I: int) returns(int) signals(overflow)
	end i_not

i_oparse = proc(S: string) returns(int) signals(bad_format, overflow)
	end i_oparse

i_or = proc(I, J: int) returns(int) signals(overflow)
	end i_or

i_ounparse = proc(I: int) returns(string)
	end i_ounparse

i_rotate = proc(I, CNT: int) returns(int) signals(overflow)
	end i_rotate

i_set = proc(I, START, CNT, VAL: int) returns(int)
	signals(bounds, illegal_size, overflow)
	% low order bit is numbered 0
	end i_set

i_set1 = proc(I, BIT: int, VAL: bool) returns(int) signals(bounds, overflow)
	% low order bit is numbered 0
	end i_set1

i_shift = proc(I, CNT: int) returns(int) signals(overflow)
	end i_shift

i_xor = proc(I, J: int) returns(int) signals(overflow)
	end i_xor

isqrt = proc(I: int) returns(int) signals(complex_result)
	end isqrt

link_file = proc(OFN, NFN: file_name) signals(not_possible(string))
    	% make NFN a link to OFN
	end link_file

open_append = proc(NAME: string) returns(stream)
	      signals(bad_format, not_possible(string))
	end open_append

open_read = proc(NAME: string) returns(stream)
	    signals(bad_format, not_possible(string))
	end open_read

open_write = proc(NAME: string) returns(stream)
	     signals(bad_format, not_possible(string))
	end open_write

quit_ = proc()
	end quit_

run_time = proc() returns(time)
	end run_time

set_connected_dir = proc(S: string) signals(bad_format, not_possible(string))
    	% change connected directory
	end set_connected_dir

sleep = proc(secs: int)
    	% sleep for SECS seconds
	end sleep

symlink_file = proc(OFN, NFN: file_name) signals(not_possible(string))
    	% make NFN a link to OFN
	end symlink_file

unify_spacing = proc(S: string) returns(string)
	end unify_spacing

unix_cmd = proc(CMD: string)
	end unix_cmd
